<?php
/*
ccu825_events - CCU825. Событие. Таблица для хранения событий контроллера ccu825.
ccu825_temp - CCU825. Температура платы. Таблица для хранения значений температуры платы контроллера ccu825.
Outputs[RELE1,RELE2,OUT1,OUT2,OUT3,OUT4,OUT5]
0 - RELE1 Free
1 - RELE2 Free
2 - OUT1 Перезагрузка Мега2561
3 - OUT2 Обогреватель в зале
4 - OUT3 Free
5 - OUT4 Free
6 - OUT5 Free
v 1.0.1 Добавил установку режима ARM/Disarm на MegaD2561.
v 1.0.2 Добавил вызов скрипта управления событиями при постановке на охрану - /var/www/cntrl_alarm_on.php.
v 1.0.3 Добавил отслеживание состояний выходов. RELE1,RELE2, OUT0-4

*/
require_once("ab-cms/class/main_class.php");
$ab = new cms_lib();

$keys_id = $ab->get_id("keys");
$ccu825_id = $ab->get_id("ccu825_events");
$ccu825_temp_id = $ab->get_id("ccu825_temp");
$keys_journal_id = $ab->get_id("keys_journal");

# Имя пользователя и пароль для доступа к ccu proxy.
$jsonHost = "localhost:8080";
$jsonUser = "admin";
$jsonPass = "XXXXXXX";

# Функция HTTP Basic аутентификации и обращения к JSON.
function http_auth_get($url,$username,$password){
	$cred = sprintf('Authorization: Basic %s',
	base64_encode("$username:$password"));
	$opts = array('http'=>array('method'=>'GET','header'=>$cred));
	$ctx = stream_context_create($opts);
	$handle = fopen ( $url, 'r', false,$ctx);

	return stream_get_contents($handle);
}

$loop = 1;
$count = 0;
#Бесконечный опрос состояния контроллера ccu825, раз в минуту.
while ( $loop == 1)
{
	$time = date("d.m.y H:i:s");
	
	$json = http_auth_get('http://'.$jsonHost.'/data.cgx?cmd={"Command":"GetStateAndEvents"}',$jsonUser,$jsonPass);
	$obj = json_decode($json, TRUE);
	
	if(empty($obj_prev)){ //При первичном запуске получаем предыдущие значения
		$obj_prev = $obj;
	}	

	# Если ЕСТЬ связь с контроллером (массив непустой), то обрабатываем массив полученный из JSON.
	if( !empty($obj) ) {
		#Состояние охраны
		$partitions = $obj['Partitions'];
		$ccuMode = $partitions['0'];
		#Внешнее питание. Состояние питания. "Off" — выключено, Число — напряжение в вольтах.
		$ccuPower = $obj['Power'];
		# Заряд и состояние аккумулятора.
		$battery = $obj['Battery'];
		$batteryCharge = $battery['Charge'];
		$batteryState = $battery['State'];
		# Температура платы CCU.
		$ccuTemp = round($obj['Temp'],1);
		# Напряжение питания CCU.
		$ccuVolt = round($obj['Power'],1);
		# Баланс на SIM карте.
		$ccuBalance = $obj['Balance'];
		# Состояние крышки корпуса
		$ccuCase = $obj['Case'];
		# Состояние выходов контроллера 2 реле + 5 выходов ОК.
		$ccuOutputs = $obj['Outputs'];
		$ccuOutputs_prev = $obj_prev['Outputs'];
	
		/*
		# Отображение полученных значений (для диагностики проблем).
		echo "Баланс: $ccuBalance \n";
		echo "Аккумулятор: $batteryState заряд: $batteryCharge% \n";
		echo "Температура на плате: $ccuTemp \n";
		echo "Режим охраны: $ccuMode\n";
		echo "Внешнее питание: $ccuPower\n";
		echo "Состояние крышки корпуса: $ccuCase\n";
		*/
		
		/* 1. Определяем факт смены режима работы CCU825 ОХРАНА/наблюдение и регистрируем событие */
		$alarm_mode = $ab->mod_read($keys_id, "#key_pio#", "", 1, "key_label='alarm'");
		$key_tmpID = $ab->mod_read($keys_id, "#tmpID#", "", 1, "key_label='alarm'");
	
		if( $ccuMode == "Arm" && $alarm_mode != 1 ){ //Постановка на охрану
			//Постановка на охрану: изменяем виртуальный ключ alarm = 1 - вкл.
			$key_pio = 1;
			$ab->mod_write($keys_id, "key_pio=$key_pio", "tmpID=$key_tmpID");
			//Устанавливаем режим Меги в ARM
			file_get_contents("http://192.168.0.14/sec/?cmd=S:1");
			//Вызываем скрипт управления событиями при постановке на охрану
			exec("/usr/bin/php /var/www/cntrl_alarm_on.php > /dev/null &");
			//Журналируем событие
			$ab->mod_write($keys_journal_id, "SYSDATE(), $key_tmpID, $key_pio");
			$ab->mod_write($ccu825_id, "'$ccuMode', SYSDATE(), ''"); 
		} else if ( $ccuMode == "Disarm" && $alarm_mode == 1 ) { // Снятие с охраны
			$key_pio = 0;
			$ab->mod_write($keys_id, "key_pio=$key_pio", "tmpID=$key_tmpID");
			//Устанавливаем режим Меги в disarm
			file_get_contents("http://192.168.0.14/sec/?cmd=S:0");
			//Журналируем событие
			$ab->mod_write($keys_journal_id, "SYSDATE(), $key_tmpID, $key_pio");
			$ab->mod_write($ccu825_id, "'$ccuMode', SYSDATE(), ''"); 
		}
		
		/* 2. Сохраняем в базу температуру платы ccu825 - 1 раз в час. */
		if ( $count % 60 == 0 ){ 
			$ab->mod_write($ccu825_temp_id, "SYSDATE(), '$ccuTemp'" );
			$count = 0;
		}
		
		/* 3. Регистрируем факт выключения/включения питания на ccu825 Power: "Off"/Число */
		if ( $ccuPower == 'Off' && is_numeric($obj_prev['Power']) ) {
			//Журналируем событие
			$ab->mod_write($ccu825_id, "'PowerOFF', SYSDATE(), '$ccuPower'");
		} else if ( is_numeric($ccuPower) && $obj_prev['Power'] == 'Off' ) {
			//Журналируем событие
			$ab->mod_write($ccu825_id, "'PowerON', SYSDATE(), '$ccuPower'");	
		}
		
		/* 4. Регистрируем события по выходам контроллера - массив Outputs[0-6] */
		$heat_pio = $ab->mod_read($keys_id, "#key_pio#", "", 1, "key_label='heat'");		//Считали состояние ключа heat в БД
		for( $i = 0; $i <= 6; $i++){
			if( $ccuOutputs[$i] != $ccuOutputs_prev[$i] && $i != 3 ) // Состояние выходов изменилось регистрируем событие.
			{
				$ab->mod_write($ccu825_id, "'Output$i', SYSDATE(), '$ccuOutputs[$i]'");
				
			} elseif ( $i == 3 && ( $ccuOutputs[$i] != $ccuOutputs_prev[$i] || $heat_pio != $ccuOutputs[$i] ) ){ //Если это реле обогревателя
				// И изменилось состояние входа или оно не равно состоянию в БД то синхронизуем ключ отопления с реальностью
				$heat_pio = $ccuOutputs[$i];
				//Журналируем смену ключа heat
				$heat_tmpID = $ab->mod_read($keys_id, "#tmpID#", "", 1, "key_label='heat'");
				$ab->mod_write($keys_id, "key_pio=$heat_pio", "tmpID=$heat_tmpID");
				$ab->mod_write($keys_journal_id, "SYSDATE(), $heat_tmpID, $heat_pio");
				$ab->mod_write($ccu825_id, "'Output$i', SYSDATE(), '$ccuOutputs[$i]'");
			}
		}
		
		/* Сохраняем полученные параметры, для регистрации событий изменения значений */
		$obj_prev = $obj;
	} else {
	# Если нет связи с сконтроллером (массив пустой)
	echo "$time: Нет связи с контроллером!\n";
	}
	
	$count++;
	#Задержка перед следующим опросом
	sleep(60);
}

?>
